#!/usr/bin/python
# -*- coding: utf-8 -*-
## Copyright (C) 2001-2008 Alberto Milone <albertomilone@alice.it>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import sys, dbus, logging, os, re
import optparse

SERVICE_NAME   = 'com.ubuntu.ScreenResolution.Mechanism'
OBJECT_PATH    = '/'
INTERFACE_NAME = 'com.ubuntu.ScreenResolution.Mechanism'

# Usage:
# python /usr/share/screen-resolution-extra/nvidia-polkit.py --help
#

def get_xkit_service(widget=None):
    '''returns a dbus interface to the screenresolution mechanism'''
    service_object = dbus.SystemBus().get_object(SERVICE_NAME, OBJECT_PATH)
    service = dbus.Interface(service_object, INTERFACE_NAME)

    return service


def main(options):
    exit_code = 1
    
    conf = get_xkit_service()
    if not conf:
        # dbus_cant_connect
        logging.error("cannot connect to dbus service")
        sys.exit(1)
    
    if options.filename and options.backup_filename:
        logging.debug('making backup of /etc/X11/xorg.conf to %s and writing %s to /etc/X11/xorg.conf' \
        % (options.backup_filename, options.filename))
        exit_code = conf.backupAndWriteXorgConf([options.backup_filename, options.filename])
    elif options.filename and not options.backup_filename:
        logging.debug('writing %s to /etc/X11/xorg.conf' % options.filename)
        exit_code = conf.writeXorgConf(options.filename)
    elif not options.filename and options.backup_filename:
        logging.debug('making backup of /etc/X11/xorg.conf to %s' % options.backup_filename)
        exit_code = conf.backupXorgConf(options.backup_filename)
    elif not options.filename and not options.backup_filename:
        logging.error("called with wrong arguments")
        return exit_code

    # All went well if exit_code == 0
    return exit_code


if __name__ == "__main__":
    parser = optparse.OptionParser()
    parser.add_option("-w", "--write-from", dest="filename",
                      help="write xorg.conf from FILE", metavar="FILE")
    parser.add_option("-b", "--backup-to", dest="backup_filename",
                      help="backup file to FILE", metavar="FILE")

    (options, args) = parser.parse_args()
    
    operation_status = main(options)

    sys.exit(operation_status)





#!/usr/bin/env python

import os, sys

if len(sys.argv) <= 1:
    os.system("xgettext -k_ -kN_ -o screen-resolution-extra.pot ../ScreenResolution/ui.py")
    print "Now edit messages.pot, save it as <locale>.po, and send it" \
        " to albertomilone@alice.it.\nThanks!\n"



#!/usr/bin/python
# -*- coding: utf-8 -*-
## Copyright 2013 Canonical Ltd.
##
## Author: Alberto Milone <alberto.milone@canonical.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import sys, dbus, logging

SERVICE_NAME   = 'com.ubuntu.ScreenResolution.Mechanism'
OBJECT_PATH    = '/'
INTERFACE_NAME = 'com.ubuntu.ScreenResolution.Mechanism'

# Usage:
# python /usr/share/screen-resolution-extra/nvidia-prime.py --help
#

def get_prime_service(widget=None):
    '''returns a dbus interface to the screenresolution mechanism'''
    service_object = dbus.SystemBus().get_object(SERVICE_NAME, OBJECT_PATH)
    service = dbus.Interface(service_object, INTERFACE_NAME)

    return service


def usage():
    sys.stderr.write("Usage: %s nvidia|intel\n" % (sys.argv[0]))


if __name__ == '__main__':
    try:
        arg = sys.argv[1]
    except IndexError:
        arg = None

    if len(sys.argv[1:]) != 1:
        usage()
        exit(1)

    if arg == 'intel' or arg == 'nvidia':
        conf = get_prime_service()
        if not conf:
            # dbus_cant_connect
            logging.error("cannot connect to dbus service")
            sys.exit(1)

        # Exit using the retcode
        try:
            sys.exit(conf.prime_select(arg))
        except dbus.exceptions.DBusException as e:
            message = '%s' % e
            sys.stderr.write(message)
            if 'AccessDeniedException' in message:
                # The user probably did not enter the password
                # or cancelled
                sys.exit(5)
            else:
                sys.exit(1)
    else:
        usage()
        sys.exit(1)



# -*- coding: utf-8 -*-
#       ui.py
#       
#       Copyright 2008 Alberto Milone <albertomilone@alice.it>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import gettext

class AbstractUI:
    '''Abstract user interface.

    This encapsulates the entire program logic and all strings, but does not
    implement any concrete user interface.
    '''
    def __init__(self):
        '''
        Initialize system.
        '''
        self.gettext_domain = 'screen-resolution-extra'

        gettext.textdomain(self.gettext_domain)

        self.init_strings()
    
    def _(self, str, convert_keybindings=False):
        '''Keyboard accelerator aware gettext() wrapper.
        
        This optionally converts keyboard accelerators to the appropriate
        format for the frontend.

        All strings in the source code should use the '_' prefix for key
        accelerators (like in GTK). For inserting a real '_', use '__'.
        '''
        # KDE compatible conversion
        #result = unicode(gettext.gettext(str), 'UTF-8')
        result = gettext.gettext(str)

        if convert_keybindings:
            result = self.convert_keybindings(result)

        return result
    
    def init_strings(self):
        '''Initialize all static strings which are used in UI implementations.'''

        self.string_permission_text = self._('Monitor Resolution Settings has detected that the virtual resolution \
must be set in your configuration file in order to apply your settings.\
\n\nWould you like Screen Resolution to set the virtual resolution for you? (Recommended)')
        
        self.string_dbus_cant_connect = self._('Could not connect to Monitor Resolution Settings DBUS service.')
        
        self.string_operation_complete = self._('Please log out and log back in again.  You will then be able to use Monitor Resolution Settings to setup your monitors')
        
        self.string_cant_apply_settings = self._('Monitor Resolution Settings can\'t apply your settings.')
        
        self.string_title = self._('Monitor Resolution Settings')
        



#!/usr/bin/python
# -*- coding: utf-8 -*-
## Copyright (C) 2001-2008 Alberto Milone <albertomilone@alice.it>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from gi.repository import Gtk
import sys, dbus, logging
from ScreenResolution import ui

SERVICE_NAME   = 'com.ubuntu.ScreenResolution.Mechanism'
OBJECT_PATH    = '/'
INTERFACE_NAME = 'com.ubuntu.ScreenResolution.Mechanism'
usage = 'python policyui.py 1024x768'

import os
import sys
from subprocess import Popen, PIPE

import xkit
from xkit import xutils, xorgparser

clean = False

def checkVirtual(virtres):
    '''See if it's necessary to set the virtual resolution'''

    source = '/etc/X11/xorg.conf'
    
    try:
        a = xutils.XUtils(source)
    except(IOError, xkit.xorgparser.ParseException):#if xorg.conf is missing or broken
        return False
    
    if len(a.globaldict['Screen']) > 0:
        # See if the virtual resolution is already there and if it's big enough
        res = None
        for screen in a.globaldict['Screen']:
            try:
                res = a.getValue('SubSection', 'virtual', 0, identifier='Display', sect="Screen", reference=None)
                if res:
                    if 'x' in res.lower():
                        res = res.lower().split('x')
                    elif ' ' in res.lower().strip():
                        res = res.lower().split(' ')
                        res = [x for x in res if x != '']
                        if len(res) == 2 and int(virtres[0]) <= int(res[0]) and int(virtres[1]) <= int(res[1]):
                            # Nothing to do, the virtual resolution is already there
                            return True
            except (xkit.xorgparser.SectionException, xkit.xorgparser.OptionException, AttributeError):
                pass
    
    return False

def get_xkit_service(widget=None):
    '''
    returns a dbus interface to the screenresolution mechanism
    '''
    service_object = dbus.SystemBus().get_object(SERVICE_NAME, OBJECT_PATH)
    service = dbus.Interface(service_object, INTERFACE_NAME)

    return service

def gui_dialog(message, parent_dialog,
                      message_type=None,
                      widget=None, page=0, 
                      broken_widget=None):
    '''
    Displays an error dialog.
    '''
    if message_type == 'error':
        message_type = Gtk.MessageType.ERROR
        logging.error(message)
    elif message_type == 'info':
        message_type = Gtk.MessageType.INFO
        logging.info(message)


    
        
    dialog = Gtk.MessageDialog(parent_dialog,
                               Gtk.DialogFlags.MODAL|Gtk.DialogFlags.DESTROY_WITH_PARENT,
                               message_type, Gtk.ButtonsType.OK,
                               message)
    
    translation = ui.AbstractUI()

    dialog.set_title(translation.string_title)

    if widget != None:
        if isinstance (widget, Gtk.CList):
            widget.select_row (page, 0)
        elif isinstance (widget, Gtk.Notebook):
            widget.set_current_page (page)
    if broken_widget != None:
        broken_widget.grab_focus ()
        if isinstance (broken_widget, Gtk.Entry):
            broken_widget.select_region (0, -1)

    if parent_dialog:
        dialog.set_position (Gtk.WindowPosition.CENTER_ON_PARENT)
        dialog.set_transient_for(parent_dialog)
    else:
        dialog.set_position (Gtk.WindowPosition.CENTER)

    ret = dialog.run ()
    dialog.destroy()    

    return ret

class BootWindow:
    optimal_virtual_resolution = ['2048', '2048']
    
    def __init__(self, resolution):
        
        translation = ui.AbstractUI()
        self.permission_text = translation.string_permission_text
        self.dbus_cant_connect = translation.string_dbus_cant_connect
        self.operation_complete = translation.string_operation_complete
        self.cant_apply_settings = translation.string_cant_apply_settings
        self.resolution = resolution
        self.question_dialog(translation.string_title, self.permission_text)

    def question_dialog(self, title, message):
        '''question_dialog(title, message)

        This is a generic Gtk MessageDialog function
        for questions.
        '''
        dialog = Gtk.MessageDialog(None,
                                   Gtk.DialogFlags.MODAL,
                                   Gtk.DialogFlags.DESTROY_WITH_PARENT,
                                   type=Gtk.MessageType.QUESTION,
                                   buttons=Gtk.ButtonsType.YES_NO)
        dialog.set_markup("<b>%s</b>" % title)
        dialog.format_secondary_markup(message)
        response = dialog.run()
        dialog.destroy()

        if response == Gtk.ResponseType.YES:
            dialog.hide()
            self.on_button1_clicked(dialog)
        elif response == Gtk.ResponseType.NO:
            self.on_button2_clicked(dialog)
        else:
            self.on_button2_clicked(dialog)

    def on_button1_clicked(self, widget, data=None):
        #self.window.hide()
        self.conf = get_xkit_service()
        if not self.conf:
            sys.exit(1)
        
        # If the required resolution is lower than the optimal virtual
        # resolution (usually the highest resolution which doesn't break
        # direct rendering) then use the latter instead.
        if int(self.resolution[0]) < int(self.optimal_virtual_resolution[0]) and \
           int(self.resolution[1]) < int(self.optimal_virtual_resolution[1]):
            self.resolution = self.optimal_virtual_resolution

        status = self.conf.setVirtual(self.resolution)
        #print 'Status', status
        if status == True:
            global clean
            clean = True
        
    def on_button2_clicked(self, widget, data=None):
        pass
        #self.window.hide()


if __name__ == "__main__":
    if len(sys.argv) > 1:
        for param in sys.argv[1:]:
            if 'x' not in param:
                sys.exit(0)    
    else:
        sys.exit(0)
    
    res = sys.argv[1]
    res = res.strip().split('x')
    if checkVirtual(res):
        sys.exit(0)
    window = BootWindow(res)

    if not clean:
        sys.exit(1)



#!/usr/bin/python

from distutils.core import setup

import subprocess, glob, os.path

from DistUtilsExtra.command import *

#mo_files = []
## HACK: make sure that the mo files are generated and up-to-date
#subprocess.call(["make", "-C", "po", "build-mo"])
#for filepath in glob.glob("po/mo/*/LC_MESSAGES/*.mo"):
#    lang = filepath[len("po/mo/"):]
#    targetpath = os.path.dirname(os.path.join("share/locale",lang))
#    mo_files.append((targetpath, [filepath]))

setup(
    name="screen-resolution-extra",
    author="Alberto Milone",
    author_email="albertomilone@alice.it",
    maintainer="Alberto Milone",
    maintainer_email="albertomilone@alice.it",
    url="http://www.albertomilone.com",
    license="gpl",
    description="Extension for the GNOME Screen Resolution applet",
    packages=["ScreenResolution"],
    data_files=[
                ("/etc/dbus-1/system.d", glob.glob("com.ubuntu.ScreenResolution.Mechanism.conf")),
                ("share/dbus-1/system-services", glob.glob("com.ubuntu.ScreenResolution.Mechanism.service")),
                ("share/polkit-1/actions", glob.glob("screenresolution-mechanism.policy")),
                ("share/screen-resolution-extra", glob.glob("screenresolution-mechanism")),
                ("share/screen-resolution-extra", glob.glob("policyui.py")),
                ("share/screen-resolution-extra", glob.glob("nvidia-polkit.py")),
                ("share/screen-resolution-extra", glob.glob("nvidia-prime.py")),
               ],
    cmdclass = { "build" : build_extra.build_extra,
            "build_i18n" :  build_i18n.build_i18n,
            "build_help" :  build_help.build_help,
            "build_icons" :  build_icons.build_icons }
)





